#!/usr/bin/perl
#==============================================================================
# A Simple Mini Gene-Finding Program
#
# Programmer:    <your name>
#
# SUMMARY:
# The Central Dogma of molecular biology relates DNA, RNA,
# and proteins. Briefly put, the DNA sequence (gene) provides instructions
#             for the protein's production;
#
# STEP ONE: A DNA sequence such as AGTCTGACCTAG
#   is TRANSCRIBED to a messenger RNA (mRNA) that looks like
#                                  UCAGACUGGAUC
#  Remember that A complements T and C complements G in DNA.
#  However, when making a complementary RNA (transcribing)
#     U (uracil) is used instead of T.
#
# STEP TWO: The mRNA is TRANSLATED into PROTEIN (strings of amino acids).
# Special translation structures (ribosomes) "read" the mRNA three bases at
# a time. Every "triplet" codes for a particular amino acid except for three
# that code for "STOP" at the end of a gene.

# IMPORTANT RULES for BUILDING your PROGRAM

# 1. The first triplet in an mRNA is "AUG" which also codes for
#         the amino acid methionine (Met).
#         The triplet AUG can be BOTH a "Start" and a simple instruction
#         for putting a methionine somewhere in the middle of a protein.
#         YOUR PROGRAM should search for Met BOTH at the beginning and elsewhere
#         and report the whereabouts.
#
# 2. The last triplet should be one of the stops, indicated by three DOTS in
#         this code (...). Every mRNA should have a stop at the end but not
#         elsewhere. YOUR PROGRAM should search for STOP both at the end and
#         elsewhere and report on the whereabouts.
#
# 3. An mRNA strand may be "read" in any of three "reading frames".  One of
#         those is often better than the other two. Better means it has a
#         proper START and STOP and no STOPS elsewhere. YOUR OUTPUT will help
#         you determine whether ANY of the frames looks like a good gene.
#
#
#         NOTE: Some of the Perl in this kit is either directly or indirectly
#             from: Dwyer, Rex A. (2003). Genomic Perl -- From Bioinformatics
#             Basics to Working Code. Cambridge University Press, Cambridge, UK.
#
# INPUT:   [you summarize this]
#
# OUTPUT:  [you summarize this]
#
# Status:  [you leave a note here that mentions if this program
#           is Working or Not Working]
#
# Date Last Modified:
#    09/26/2007 (mdl) -- set up Starter Kit with Rex Dwyer's subroutines
#    09/27/2007 (bdd) -- checked biological relevance and terms; added some code
#
#==============================================================================

use strict;         # request conservative error checking
use warnings;

# --------------------- DECLARE VARIABLES ------------------------------

my %codonMap;       # declare a hash table to hold the Amino Acid abbreviations
                    # corresponding to each possible triple of DNA (codon).
                    
my $DNA;            # will hold the DNA sequence to search

my $RNA1;            # will hold the RNA sequence for the first reading frame
my $RNA2;            # will hold the RNA sequence for the second reading frame
my $RNA3;            # will hold the RNA sequence for the third reading frame

my $RNALength1;     # the length of the RNA string in the first reading frame
my $RNALength2;     # the length of the RNA string in the second reading frame
my $RNALength3;     # the length of the RNA string in the third reading frame

my $protein;        #

my $locationMet;
my $locationMiddleMet;
my $locationSTOP;
my $locationWrongSTOP;

#-----------------------------------------------------------------------

# set up a hash table that will map codons (three letter sequences) of DNA
# to their respective Amino Acid (AA) equivalent
%codonMap = getAminoAcidLookUpTable();

# get the DNA sequence stored in a Perl string in either one of
# two ways:  (1) make up a string when you are testing

$DNA = "TATACATGCGTTTACTATAGCATTTAAGGATT";

print "\t\t DNA: \t $DNA \n";

# --------- TRANSCRIBE --------------------------
print "\t\t\t\t (-- transcribe --) \n";
$RNA1 = transcribe($DNA);
print "\t\t RNA: \t $RNA1 \n";

print "\n==============================================================\n";

# --------- TRANSLATE Reading Frame #1 --------------------------
$RNALength1 = length($RNA1);
print "RNA length: \t $RNALength1 \n";

print "\t\t RNA: \t\t $RNA1 \n";

print "\t\t\t\t\t (-- translate --) \n";
$protein = translate($RNA1);
print "Reading Frame (1): \t ", $protein, "\n\n";

# you put in IF-ELSE statements HERE to locate and print the whereabouts
# of the starting Met (or not), STOP at end (or not), Met other than
# the starting location (or not), and STOP other than at the end (or not)





print "==============================================================\n";


# --------- TRANSLATE Reading Frame #2 --------------------------
$RNA2 = $RNA1;
$RNA2 =~ s/.//;    # REMOVE the first RNA nucleotide
                   # s (substitute) any-first-char for nothing
                   
# now, look at the code above in Reading Frame #1
# copy(reuse) some of those statements here ...



# you put in IF-ELSE statements HERE to locate and print the whereabouts
# of the starting Met (or not), STOP at end (or not), Met other than
# the starting location (or not), and STOP other than at the end (or not)





print "==============================================================\n";



# --------- TRANSLATE Reading Frame #3 --------------------------

# now, look at the code above in Reading Frame #2
# copy(reuse) some of those statements here ...




# you put in IF-ELSE statements HERE to locate and print the whereabouts
# of the starting Met (or not), STOP at end (or not), Met other than
# the starting location (or not), and STOP other than at the end (or not)





print "==============================================================\n";



# ---------------------- end of your Perl -----------------------------------




## ============================================================================
##              Do NOT edit or erase the lines below this point!!
## ============================================================================

#-------------------- Subroutines for you to use -----------------------
#
# These subroutines (sometimes called functions) below are ready for you to
# use in your program. Think of these routines as "buttons on your
# calculator" ... they are ready to use. When you "use" one of these
# routines, we say "your program is CALLING" for the use of that routine.
#
# The subroutines getAminoAcidLookUpTable(), transcribe() and translate() are
# Copyright (c) 2002 Rex A. Dwyer.
#-----------------------------------------------------------------------

#------------------------\
# getAminoAcidLookUpTable \
#-----------------------------------------------------------------------
# Construct hash that maps codons to amino acids by reading table
# from DATA lines at the end of the program, which have the form:
# Residue Codon1 Codon2 ...
sub getAminoAcidLookUpTable
{
  my %codonMap;

  while (my $in = <DATA> )    # assigns next line of DATA to $in; fails if none
  {
    chomp($in);                   # remove line feed at end of $in
    my @codons = split " ",$in;
    my $residue = shift @codons;  # remove first item from @codons and assign
    foreach my $nnn (@codons) {
        $codonMap{$nnn} = $residue;
    }
  }

  return %codonMap;;
}
#-----------------------------------------------------------------------



#-----------\
# transcribe \
#-----------------------------------------------------------------------
#  Software transcriptase: turns a DNA string into an RNA string.
#  RETURNS: the RNA string.
sub transcribe
{
    my ($dna) = @_;  # the DNA string to be transcribed
    
    my $rna = $dna;
    $rna =~ tr/ACGT/UGCA/;
    return $rna;
}
#-----------------------------------------------------------------------


#-----------\
# translate  \
#-----------------------------------------------------------------------
# Software ribosome: translates mRNA strings to proteins
# RETURNS: the protein string. (3-letter abbreviations.)
sub translate
{
    my ($mrna) = @_;  # the mRNA string to be translated.
    my $pro = "";
    while ( $mrna =~ s/(...)// ) {
        $pro = $pro . $codonMap{$1};
    }
    return $pro;
}
#-----------------------------------------------------------------------

## The lines below are not perl statements and are not executed as part of the 
## program.  Instead, they are available to be read as input by the program
## using the file handle DATA.
__END__
Ala GCU GCC GCA GCG
Arg CGU CGC CGA CGG AGA AGG
Asn AAU AAC
Asp GAU GAC 
Cys UGU UGC
Gln CAA CAG
Glu GAA GAG
Gly GGU GGC GGA GGG
His CAU CAC
Ile AUU AUC AUA
Leu UUA UUG CUU CUC CUA CUG
Lys AAA AAG
Met AUG
Phe UUU UUC
Pro CCU CCC CCA CCG
Ser UCU UCC UCA UCG AGU AGC
Thr ACU ACC ACA ACG
Trp UGG
Tyr UAU UAC
Val GUU GUC GUA GUG
... UAA UAG UGA
